export const rem = (baseSize = 12) => {
  const flexibleEl = document.querySelector('meta[name="flexible"]')
  const docEl = document.documentElement
  let metaEl = document.querySelector('meta[name="viewport"]')
  let dpr = 0
  let scale = 0

  if (metaEl) {
    const match = metaEl.getAttribute('content').match(/initial\-scale=([\d\.]+)/)
    if (match) {
      scale = parseFloat(match[1])
      dpr = parseInt(1 / scale, 10)
    }
  } else if (flexibleEl) {
    const content = flexibleEl.getAttribute('content')
    if (content) {
      const initialDpr = content.match(/initial\-dpr=([\d\.]+)/)
      const maximumDpr = content.match(/maximum\-dpr=([\d\.]+)/)
      if (initialDpr) {
        scale = parseFloat(initialDpr[1])
        dpr = parseInt(1 / scale, 10)
      }
      if (maximumDpr) {
        scale = parseFloat(maximumDpr[1])
        dpr = parseInt(1 / scale, 10)
      }
    }
  }

  if (!dpr && !scale) {
    // const isAndroid = window.navigator.appVersion.match(/android/gi)
    const isIPhone = window.navigator.appVersion.match(/iphone/gi)
    const { devicePixelRatio } = window
    if (isIPhone) {
      // iOS下，对于2和3的屏，用2倍的方案，其余的用1倍方案
      if (devicePixelRatio >= 3 && (!dpr || dpr >= 3)) {
        dpr = 3
      } else if (devicePixelRatio >= 2 && (!dpr || dpr >= 2)) {
        dpr = 2
      } else {
        dpr = 1
      }
    } else {
      // 其他设备下，仍旧使用1倍的方案
      dpr = 1
    }
    scale = 1 / dpr
  }

  docEl.setAttribute('data-dpr', dpr)

  if (!metaEl) {
    metaEl = document.createElement('meta')
    metaEl.setAttribute('name', 'viewport')
    metaEl.setAttribute('content', `initial-scale=${scale}, maximum-scale=${scale}, minimum-scale=${scale}, user-scalable=no`)
    if (docEl.firstElementChild) {
      docEl.firstElementChild.appendChild(metaEl)
    } else {
      const wrap = document.createElement('div')
      wrap.appendChild(metaEl)
      document.write(wrap.innerHTML)
    }
  }

  const setHtmlFontsize = function () {
    let { width } = docEl.getBoundingClientRect()
    if (width / dpr > 540) {
      width = 540 * dpr
    }
    const _rem = width / 10
    docEl.style.fontSize = `${_rem}px`
  }
  setHtmlFontsize()

  // detect 0.5px supports
  // if (dpr >= 2) {
  //   var fakeBody = document.createElement('body')
  //   var testElement = document.createElement('div')
  //   testElement.style.border = '.5px solid transparent'
  //   fakeBody.appendChild(testElement)
  //   docEl.appendChild(fakeBody)
  //   if (testElement.offsetHeight === 1) {
  //     docEl.classList.add('hairlines')
  //   }
  //   docEl.removeChild(fakeBody)
  // }

  if (document.readyState === 'complete') {
    // 如果是异步加载js的话，这里不会产生效果
    document.body.style.fontSize = `${baseSize * dpr}px`
  } else {
    document.addEventListener('DOMContentLoaded', () => {
      document.body.style.fontSize = `${baseSize * dpr}px`
    }, false)
  }
}
